;________________________________________________________________________
;
;                            P C D 3 3 1 2 . A S M
;________________________________________________________________________
;
; Sending packet audio tones with a PCD3312 I2C chip
; by Byon Garrabrant  N6BG  byon@mail.com  Aug 98
;________________________________________________________________________
;
	cblock				; variables
		FLAGS			; generic flags / bits
		next_freq
		intWsave		; interrupt flags
		intSTATUSsave
		intPCLATHsave
	endc
;________________________________________________________________________

#define	PCD3312_ADDRESS		H'48'	; addreess plus write mode
#define	PCD3312_OFF		H'1'
#define	PCD3312_1200		H'26'
#define	PCD3312_2200		H'27'

#define	SENT			FLAGS, 1
#define	BANK1			H'7F'
;________________________________________________________________________

	ORG	H'0004'

Timer_Interrupt
	movwf	intWsave		; save W
	swapf	STATUS, W		; save STATUS
	movwf	intSTATUSsave
	movfw	PCLATH			; save PCLATH
	movwf	intPCLATHsave

	bcf	STATUS, RP0		; select bank 0 (working with TMR0)
	movlw	D'130'-2		; bump timer forward for
	subwf	TMR0, F			; update TMR0 for next int
	bsf	SENT			; ready for the next bit

timer_end
	bcf	INTCON, T0IF		; enable TMR0 int
	movfw	intPCLATHsave		; restore PCLATH
	movwf	PCLATH			
	swapf	intSTATUSsave, W	; restore STATUS
	movwf	STATUS
	swapf	intWsave, F		; restore W
	swapf	intWsave, W
	retfie				; return from interrupt

;________________________________________________________________________

	include	"I2C.ASM"		; include I2C code
;________________________________________________________________________

Tone_Init
	clrf	INTCON
	call	I2C_Init

Tone_Disable
	bcf	INTCON, T0IE		; disableTMR0int
	bsf	STATUS, RP0		; Select Bank 1
	movlw	b'11010111'		; divide TMR0 by 256
	movwf	OPTION_REG & BANK1	; enable TMR0
	bcf	STATUS, RP0		; Select Bank 0
	bcf	INTCON, GIE		; disable ints

	call	I2C_Start		; send request for off
	movlw	PCD3312_ADDRESS
	call	I2C_Send_Byte
	movlw	PCD3312_OFF
	call	I2C_Send_Byte
	call	I2C_Stop

	return

Tone_Enable
	bsf	STATUS, RP0		; Select Bank 1
	movlw	b'11010011'		; div by 16
	movwf	OPTION_REG & BANK1	; enable TMR0
	bcf	STATUS, RP0		; Select Bank 0
	bsf	INTCON, T0IE		; enableTMR0int
	bsf	INTCON, GIE		; enable ints
	movlw	PCD3312_1200		; set initial freq
	movwf	next_freq
	return
;________________________________________________________________________

Tone_Toggle_If_No_Carry
	btfss	SENT			; wait for last bit to be sent
	 goto	$-1

	bcf 	SENT			; next bit is ready

	skpnc				; if sending a 1
	 return				; leave

	movfw	next_freq
	sublw	PCD3312_1200 + PCD3312_2200	; change freq
	movwf	next_freq

	call	I2C_Start		; send request for next bit
	movlw	PCD3312_ADDRESS
	call	I2C_Send_Byte
	movfw	next_freq
	call	I2C_Send_Byte
	call	I2C_Stop
	return
;________________________________________________________________________
